#include "Delay10us.h"

/*
** ===================================================================
**     Method      :  Cpu_Delay10US (component 56F8006_32_LQFP)
**
**     Description :
**         This method realizes software delay. The length of delay
**         is at least 10 microsecond multiply input parameter
**         [us10]. As the delay implementation is not based on real
**         clock, the delay time may be increased by interrupt
**         service routines processed during the delay. Adding read
**         wait states for external program memory can cause delay
**         extension as well. The method is independent on selected
**         speed mode.
**     Parameters  :
**         NAME            - DESCRIPTION
**         us100           - Number of 10 us delay repetitions.
**     Returns     : Nothing
** ===================================================================
*/
/*lint -save -e???? Disable MISRA rule (all) checking. */
asm void Cpu_Delay10US(word us10)
{
  /* Total irremovable overhead: about 16 cycles */
  /* move.w: 2 cycles overhead (load parameter into register) */
  /* jsr:    5 cycles overhead (jump to subroutine) */
  /* rts:    8 cycles overhead (return from subroutine) */
  /* nop:    1 cycles overhead (aditional nops) */

  loop:
  /* 100 us delay block begin */
  /*
   * Delay
   *   - requested                  : 100 us @ 4MHz,
   *   - possible                   : 400 c, 100000 ns
   *   - without removable overhead : 394 c, 98500 ns
   */
  adda #2, SP                          /* (1 c: 250 ns) move SP forward */
  move.l A10, X:(SP)                   /* (2 c: 500 ns) push A */
  move.w #22, A                        /* (2 c: 500 ns) number of iterations */
  do A, label0                         /* (8 c: 2000 ns) repeat 37x nop */
    nop                                /* (1 c: 250 ns) wait for 1 c */
  label0:
  move.l X:(SP), A                     /* (2 c: 500 ns) pop A */
  suba #2, SP                          /* (1 c: 250 ns) move SP back */
  /* 100 us delay block end */
  dec.w Y0                             /* us10 parameter is passed via Y0 register */
  jne loop                             /* next loop */
  nop                                  /* avoid pipeline conflicts */
  rts                                  /* return from subroutine */
}
/*lint -restore */
